<?php

namespace Ems\Services;

use Ems\Repository\{
    CertisignRepository,
    OrganizacaoRepository,
    EstruturaRepository
};
use Ems\Util\{
    Base64Util,
    ExpectedException,
    RequestUtils,
    Utils
};
use GuzzleHttp\Client;
use Util\{
    Environment,
    NovoCodigo,
    DateUtil
};
use Ems\Services\{
    ParamOrgService,
    ParceiroNegocioService,
    PessoaService,
    OperadorService,
    ReportService,
    VinculoService,
    DocumentoService
};
use EMS\Util\Utilities;

class CertisignService {
    /** @var Environment */
    protected $environment;

    /** @var CertisignRepository */
    protected $certisignRepository;

    /** @var ParamOrgService */
    protected $paramOrgService;

    /** @var Client */
    protected $http;

    protected $accountInfo = null;

    /** @var string */
    protected $certisignUrl = null;

    /** @var ParceiroNegocioService */
    protected $parceiroNegocioService;

    /** @var OrganizacaoRepository */
    protected $organizacaoRepository;

    /** @var EstruturaRepository */
    protected $estruturaRepository;

    /** @var PessoaService */
    protected $pessoaService;

    /** @var OperadorService */
    protected $operadorService;

    /** @var ReportService */
    protected $reportService;

    /** @var VinculoService */
    protected $vinculoService;

    /** @var  DocumentoService */
    protected $documentoService;


    /** @var Utilities */
    protected $utilities;

    public function __construct(
        Environment         $environment,
        CertisignRepository $certisignRepository,
        ParamOrgService     $paramOrgService,
        string              $certisignUrl,
        ParceiroNegocioService  $parceiroNegocioService,
        OrganizacaoRepository   $organizacaoRepository,
        EstruturaRepository     $estruturaRepository,
        PessoaService           $pessoaService,
        OperadorService         $operadorService,
        ReportService           $reportService,
        VinculoService          $vinculoService,
        DocumentoService        $documentoService
    ) {
        $this->environment          = $environment;
        $this->certisignRepository  = $certisignRepository;
        $this->paramOrgService      = $paramOrgService;
        $this->certisignUrl         = $certisignUrl;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->organizacaoRepository    = $organizacaoRepository;
        $this->estruturaRepository    = $estruturaRepository;
        $this->pessoaService            = $pessoaService;
        $this->operadorService          = $operadorService;
        $this->reportService            = $reportService;
        $this->vinculoService           = $vinculoService;
        $this->documentoService         = $documentoService;

        $nrOrg = $this->environment->getNrOrg();

        if (!$nrOrg) {
            $this->setNrOrgFromHeader();
        }

        $this->utilities = Utils::getInstance('Utilities');
    }

    public function getAccountInfo() {
        if ($this->accountInfo !== null) {
            return $this->accountInfo;
        }
        $paramOrg = $this->paramOrgService->get();

        $paramsAreSet = isset($paramOrg['DSTOKENAELETRON'])
            && isset($paramOrg['NMREMAELETRON'])
            && isset($paramOrg['DSEMAILAELETRON'])
            && isset($paramOrg['NRCPFREMAELETRON'])
            && isset($paramOrg['IDTIPOASSINATURACERTSIGN'])
            && isset($paramOrg['NRTIPODOCCERTSIGN']);

        if (!$paramsAreSet) {
            return null;
        }

        $accountInfo = [
            'token' => $paramOrg['DSTOKENAELETRON'],
            'name' => $paramOrg['NMREMAELETRON'],
            'email' => $paramOrg['DSEMAILAELETRON'],
            'cpf' => $paramOrg['NRCPFREMAELETRON'],
            'idtipoassinatura' => $paramOrg['IDTIPOASSINATURACERTSIGN'],
            'nrtipodocumento' => $paramOrg['NRTIPODOCCERTSIGN']
        ];
        
        return $accountInfo;
    }

    public function getUri(
        string $route
    ): string {
        $uri = $this->certisignUrl . $route;
        return $uri;
    }

    public function upload(
        string $fileName,
        string $base64
    ): string {
        $accountInfo = $this->getAccountInfo();

        $uri = $this->getUri('document/upload');
        $incompleteBase64 = Base64Util::getIncompleteBase64($base64);

        $headers = [
            'Content-Type' => 'application/json',
            'Token' => $accountInfo['token'],
        ];

        $body = [
            'fileName' => $fileName,
            'bytes' => $incompleteBase64,
        ];

        $responseBody = RequestUtils::request($uri, $body, 'POST', $headers);
        $uploadId = $responseBody['uploadId'] ?? null;

        if (!$uploadId) {
            throw new ExpectedException('Não foi possível fazer o upload do arquivo no Certisign!');
        }

        return $uploadId;
    }

    public function delete(
        string $id
    ) {
        $accountInfo = $this->getAccountInfo();

        $uri = $this->getUri("document/delete/$id");

        $headers = [
            'Token' => $accountInfo['token'],
        ];

        $body = null;

        $responseBody = RequestUtils::request($uri, $body, 'DELETE', $headers);
        $code = $responseBody['code'] ?? null;
        $message = $responseBody['message'] ?? null;

        if (!$code) {
            return;
        }

        throw new ExpectedException("(Certisign) $message");
    }

    public function create(
        array $options
    ) {
        $accountInfo = $this->getAccountInfo();

        $documentName = $options['documentName'] ?? null;
        $uploadId = $options['uploadId'] ?? null;
        $uploadName = $options['uploadName'] ?? null;
        $signers =  $options['signers'] ?? [];
        $eSigners =  $options['electronicSigners'] ?? [];

        $uri = $this->getUri('document/create');

        $headers = [
            'Content-Type' => 'application/json',
            'Token' => $accountInfo['token'],
        ];

        $body = [
            'document' => [
                'name' => $documentName,
                'upload' => [
                    'id' => $uploadId,
                    'name' => $uploadName,
                ],
            ],
            'sender' => [
                'name' => $accountInfo['name'],
                'email' => $accountInfo['email'],
                'individualIdentificationCode' => $accountInfo['cpf'],
            ],
            'signers' => $signers,
            'electronicSigners' => $eSigners,
            'signatureStandard' => $accountInfo['idtipoassinatura'],
            'typeId' => $accountInfo['nrtipodocumento']
        ];
        
        $responseBody = RequestUtils::request($uri, $body, 'POST', $headers);
        $code = $responseBody['code'] ?? null;
        $message = $responseBody['message'] ?? null;

        if ($code) {
            throw new ExpectedException("(Certisign) $message");
        }

        return $responseBody;
    }

    public function validateSignatures(
        string $documentKey
    ) {
        $accountInfo = $this->getAccountInfo();

        $uri = $this->getUri("document/ValidateSignatures/?key=$documentKey");

        $headers = [
            'Token' => $accountInfo['token'],
        ];

        $body = null;

        $responseBody = RequestUtils::request($uri, $body, 'GET', $headers);
        $code = $responseBody['code'] ?? null;
        $message = $responseBody['message'] ?? null;

        if ($code) {
            throw new ExpectedException("(Certisign) $message");
        }

        return $responseBody;
    }

    public function callback(
        array $body
    ) {
        $action = $body['action'];

        $actionsSignature = [
            'SIGNATURE-ELETRONIC',
            'SIGNATURE-DIGITAL',
            'SIGNATURE-SERVER',
        ];

        $actionsRejection = [
            'SIGNATURE-NOK',
        ];

        $isSignatureAction = in_array($action, $actionsSignature);
        $isRejectionAction = in_array($action, $actionsRejection);

        if ($isSignatureAction) {
            $this->documentOnSignature($body);
        } else if ($isRejectionAction) {
            $this->documentOnRejection($body);
        }
    }

    private function setNrOrgFromHeader() {
        $headers = getallheaders();

        $clientSecret = $headers['authentication'] ?? null;

        if ($clientSecret === null) {
            var_dump('Token de autenticação não está presente!', $headers);die;
        }

        $organizacao = $this->organizacaoRepository->getOrganizacaoByClientSecret($clientSecret, true);
        $nrOrg = (int) $organizacao['NRORG'];

        $this->environment->setNrOrg($nrOrg);
    }

    public function documentOnSignature(
        array $body
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $documentId = $body['documentId'];
        $identifier = $body['identifier'];

        $parcNegocio = $this->parceiroNegocioService->getParceiroNegocioByNrInscricao(
            $identifier
        );

        if (!$parcNegocio) {
            throw new ExpectedException("Não foi encontrado o parceiro de negócio com número de inscrição $identifier");
        }

        $nrParcNegocio = $parcNegocio['NRPARCNEGOCIO'];

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARCNEGOCIO' => $nrParcNegocio,
            'P_IDDOCUMENTO' => $documentId,
        ];

        $this->certisignRepository->setDocumentAsSigned($params);
    }

    public function documentOnRejection(
        array $body
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $documentId = $body['documentId'];
        $identifier = $body['identifier'];
        $message = $body['message'];

        $parcNegocio = $this->parceiroNegocioService->getParceiroNegocioByNrInscricao(
            $identifier
        );

        if (!$parcNegocio) {
            throw new ExpectedException("Não foi encontrado o parceiro de negócio com número de inscrição $identifier");
        }

        $nrParcNegocio = $parcNegocio['NRPARCNEGOCIO'];

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARCNEGOCIO' => $nrParcNegocio,
            'P_IDDOCUMENTO' => $documentId,
            'P_DSREJEICAO' => $message,
        ];

        $this->certisignRepository->setDocumentAsRejected($params);
    }

    public function saveAssinaturaEletronica(
        array $row
    ): void {
        $fileName = $row['NMFILE'];
        $dataUrl = $this->getDataUrlFromRow($row);
        $row['FILE'] = $dataUrl;

        $dsPath = $this->saveArquivoFileServer($row);
        $uploadId = $this->upload($fileName, $dataUrl);

        $documentOptions = $this->getDocumentOptions($row, $uploadId);
        $createResponse = $this->create($documentOptions);

        $integracaoRow = $this->getIntegracaoRow($row, $createResponse, $dsPath);
        $this->insertIntegracao($integracaoRow);
    }

    public function getDataUrlFromRow(
        array $row
    ): string {
        $fileStr = $row['FILE'];
        $isHttpUrl = strpos($fileStr, 'http') === 0;

        if (!$isHttpUrl) {
            return $fileStr;
        }

        $file = $this->reportService->downloadReport($fileStr);
        $base64 = $file['base64'];
        $type = $file['type'];

        $dataUrl = Base64Util::base64ToDataUrl($base64, $type);

        return $dataUrl;
    }

    public function saveArquivoFileServer(
        array $row
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $base64 = (string) $row['FILE'];

        $directory = "EMS/$nrOrg/Certisign/";
        $name = $row['NMFILE'];

        $filePath = $this->utilities->saveFileOnFileServer(
            $base64, null, $nrOrg, $directory,
            $name, null, false
        );

        return $filePath;
    }

    public function getDocumentOptions(
        array $row,
        string $uploadId
    ): array {
        $fileName = $row['NMFILE'];
        $nrParcNegocio = $row['NRPARCNEGOCIO'];

        $parcNegocio = $this->parceiroNegocioService->get($nrParcNegocio);
        $name = $parcNegocio['NMPRINCIPALPARC'];
        $cpf = $parcNegocio['NRINSCRICAOPARC'];
        $email = $this->getEmailSignatario($nrParcNegocio);

        $signer = [
            'step' => 1,
            'title' => 'Signatário',
            'name' => $name,
            'email' => $email,
            'individualIdentificationCode' => $cpf,
        ];

        $documentOptions = [
            'documentName' => $row['NMDOCUMENTO'],
            'uploadId' => $uploadId,
            'uploadName' => $fileName,
            'signers' => [],
            'electronicSigners' => [],
        ];

        if ($row['IDTIPOASSINATURA'] === 'ELETRONICA') {
            $documentOptions['electronicSigners'][] = $signer;
        } else {
            $documentOptions['signers'][] = $signer;
        }

        return $documentOptions;
    }

    public function getEmailSignatario(
        int $nrParcNegocio
    ): string {
        $emailComunicaParc = $this->parceiroNegocioService->getEmailParcNegocio($nrParcNegocio, false);

        if ($emailComunicaParc) {
            return $emailComunicaParc;
        }

        $operador = $this->operadorService->getByNrParcNegocio($nrParcNegocio, false);
        $emailOperador = $operador['DSEMAILOPER'] ?? null;

        if (is_null($emailOperador)) {
            throw new \Ems\Util\ExpectedException('Não foi encontrado nenhum e-mail do parceiro de negócio ' .$nrParcNegocio);
        }

        return $emailOperador;
    }

    public function getIntegracaoRow(
        array $row,
        array $createResponse,
        string $dsPath
    ): array {
        // Função chamada apenas para a adição da informação do tipo de documento na row
        // Caso o campo seja trazido diretamente pelo HCM futuramente, retirar esta função e retirar o campo do $rowFromResponse
        $accountInfo = $this->getAccountInfo();
        
        $rowFromResponse = [
            'DSIDDOCUMENTO' => $createResponse['id'],
            'DSCHAVEDOCUMENTO' => $createResponse['chave'],
            'NRIDSIGNATARIO' => $createResponse['attendees'][0]['signerId'],
            'DSLINKASSINAT' => $createResponse['attendees'][0]['signUrl'],
            'DSPATH' => $dsPath,
            'NRTIPODOCUMENTO' => $accountInfo['nrtipodocumento']
        ];

        $integracaoRow = array_merge($row, $rowFromResponse);
        return $integracaoRow;
    }

    public function insertIntegracao(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $defaultOperador = (string) $this->environment->getCdOperador();
        $currentTime = DateUtil::getDataAtualString();

        $params = [];

        $params['NRINTELETRDIGIT'] =
            $row['NRINTELETRDIGIT'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_INTELETRDIGIT', 1, 10, true);

        $params['NRORG']                = $nrOrg;
        $params['NRPARCNEGOCIO']        = $row['NRPARCNEGOCIO'];
        $params['NRTIPODOCUMENTO']      = $row['NRTIPODOCUMENTO'];
        $params['NRCATEGDOCUMENTO']     = $row['NRCATEGDOCUMENTO'];
        $params['IDSTATUS']             = $row['IDSTATUS'] ?? 'P';
        $params['DSIDDOCUMENTO']        = $row['DSIDDOCUMENTO'] ?? null;
        $params['DSLINKASSINAT']        = $row['DSLINKASSINAT'] ?? null;
        $params['NRANEXO']              = $row['NRANEXO'] ?? null;
        $params['DSPATH']               = $row['DSPATH'] ?? null;
        $params['DTTRANSMISSAO']        = $row['DTTRANSMISSAO'] ?? $currentTime;
        $params['DTASSINATURA']         = $row['DTASSINATURA'] ?? null;
        $params['IDATIVO']              = 'S';
        $params['DTINCLUSAO']           = $currentTime;
        $params['DTULTATU']             = $currentTime;
        $params['NRORGINCLUSAO']        = $nrOrg;
        $params['CDOPERINCLUSAO']       = $row['CDOPERADOR'] ?? $defaultOperador;
        $params['NRORGULTATU']          = $nrOrg;
        $params['CDOPERULTATU']         = $row['CDOPERADOR'] ?? $defaultOperador;
        $params['DTREJEICAO']           = $row['DTREJEICAO'] ?? null;
        $params['DSREJEICAO']           = $row['DSREJEICAO'] ?? null;
        $params['DSCHAVEDOCUMENTO']     = $row['DSCHAVEDOCUMENTO'] ?? null;
        $params['NMDOCUMENTO']          = $row['NMDOCUMENTO'] ?? null;
        $params['IDTIPOASSINATURA']     = $row['IDTIPOASSINATURA'] ?? null;
        $params['NRIDSIGNATARIO']       = $row['NRIDSIGNATARIO'] ?? null;
        $params['NMFILE']               = $row['NMFILE'] ?? null;

        $this->certisignRepository->insertIntegracao($params);

        return $params['NRINTELETRDIGIT'];
    }

    public function deleteAssinaturaEletronica(
        int $nrIntegracao
    ) {
        $integracao = $this->get($nrIntegracao);
        $idStatus = $integracao['IDSTATUS'];
        $idDocumento = $integracao['DSIDDOCUMENTO'];

        $isSigned = $idStatus === 'A';
        $isRejected = $idStatus === 'R';

        if ($isSigned) {
            $dtAssinatura = $integracao['DTASSINATURA'];

            throw new ExpectedException('Não é possível excluir documentos assinados! O documento foi assinado em ' . $dtAssinatura );
        }

        if ($isRejected) {
            $dtRejeicao = $integracao['DTREJEICAO'];

            throw new ExpectedException('Não é possível excluir documentos rejeitados! O documento foi rejeitado em ' . $dtRejeicao );
        }


        $this->delete($idDocumento);
        $this->deleteIntegracao($nrIntegracao);
    }

    public function get(
        int $nrIntegracao,
        $throwException = true
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $integracao = $this->certisignRepository->get($nrOrg, $nrIntegracao);

        if (!$integracao && $throwException) {
            throw new ExpectedException('Não foi encontrada a integração Certisign ' . $nrIntegracao);
        }

        return $integracao;
    }

    public function deleteIntegracao(
        int $nrIntegracao
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $this->certisignRepository->deleteIntegracao($nrOrg, $nrIntegracao);
    }

    public function downloadAssinaturaEletronica(
        int $nrIntegracao
    ) {
        $integracao = $this->get($nrIntegracao);
        $dsPath = $integracao['DSPATH'];

        $file = $this->utilities->downloadFile($dsPath);
        return $file;
    }

    public function getDocumentosPendentesAssinatura(
        int $nrVinculo, $nrperiodoapuracao
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $nrParcNegocio = $this->vinculoService->getNrParcNegocioVinculo(
            $nrVinculo
        );

        $estruturas = $this->estruturaRepository->getEstruturaDoGestor($nrOrg, $nrParcNegocio);

        $resultados = [];

        foreach ($estruturas as $estrutura) {
            $nrestruturam = $estrutura['NRESTRUTURAM'];

            $vinculoGestor = $this->vinculoService->getVinculosGestor($nrestruturam, $nrperiodoapuracao);

            $nrvinculomVinculosGestor = array_map(function ($vinculo) {
                return [
                    'NRVINCULOM' => (int) $vinculo['NRVINCULOM'],
                    'NRPARCNEGOCIO' => $vinculo['NRPARCNEGOCIO'],
                ];
            }, $vinculoGestor);

            foreach ($nrvinculomVinculosGestor as $vinculo) {
                $paramsByGestor = [
                    'P_NRORG' => $nrOrg,
                    'P_NRVINCULO' => $vinculo['NRVINCULOM'],
                    'P_NRPARCNEGOCIO' => $vinculo['NRPARCNEGOCIO']
                ];

                $relatoriosGestor = $this->certisignRepository->getDocsPendentesAssiVinculosDoGestor($paramsByGestor);

                if (!empty($relatoriosGestor)) {
                    $resultados = array_merge($resultados, $relatoriosGestor);
                }
            }
        }

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRVINCULO' => $nrVinculo,
            'P_NRPARCNEGOCIO' => $nrParcNegocio
        ];

        $relatoriosVinculo = $this->certisignRepository->getDocumentosPendentesAssinaturaVinculo(
            $params
        );

        if (!empty($relatoriosVinculo)) {
            $resultados = array_merge($resultados, $relatoriosVinculo);
        }


        return $resultados;
    }

    public function changeStatusAssinatura(
        $row
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $nrIntegracao = $row['NRINTELETRDIGIT'];
        $isAceite = $row['ISACEITE'];
        $isAceiteGestor =  $row['ISACEITEGESTOR'];
        $nrVinculo = $row['NRVINCULOM'];
        $cdOperador = $row['CDOPERADOR'];
        $file = $row['FILE'];
        $dsRejeicao = isset($row['DSREJEICAO']) ? $row['DSREJEICAO'] : null;

        $nrParcNegocio = $this->vinculoService->getNrParcNegocioVinculo(
            $nrVinculo
        );

        $categoria = $this->documentoService->getCategEspecifica(DocumentoService::CATEG_ASSINATURA);
        $nrCategoria = $categoria['NRCATEGDOCUMENTO'] ?? null;

        $assinaturaVinculo = $this->documentoService->getDoumentoByCategory($nrParcNegocio, $nrCategoria);

        if(!$assinaturaVinculo) {
            throw new ExpectedException('Não foi encontrada nenhuma assinatura para este vínculo. É necessário que haja uma assinatura antes de prosseguir.');
        }

        $paramsAssi = [
            'P_NRORG'           => $nrOrg,
            'P_NRPARCNEGOCIO'   => $nrParcNegocio,
            'P_NRINTELETRDIGIT' => $nrIntegracao
        ];

        $relatorio = $this->certisignRepository->getAssinaturaRelatorio(
            $paramsAssi
        );

        $nrAnexo = $relatorio['NRANEXO'];
        $nmAnexo = $relatorio['NMFILE'];


        if($isAceite == 'S') {
            $this->aprovaAssinatura($nrIntegracao, $nrParcNegocio, $cdOperador, $file, $nrAnexo, $nmAnexo, $isAceiteGestor);
        } else {
            $this->reprovaAssinatura($nrIntegracao, $nrParcNegocio, $cdOperador, $isAceiteGestor, $dsRejeicao);
        }

    }

    public function aprovaAssinatura(
        $nrIntegracao, $nrParcNegocio, $cdOperador, $file, $nrAnexo, $nmAnexo, $isAceiteGestor
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $rowAnexo = [
            'NRORG'         => $nrOrg,
            'FILE'          => $file,
            'NRANEXO'       => $nrAnexo,
            'NMANEXO'       => $nmAnexo,
            'CDOPERADOR'    => $cdOperador,
            'NRPARCNEGOCIO' => $nrParcNegocio
        ];


        $dsPath = $this->documentoService->updateDocumentoAssinatura($rowAnexo);

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRPARCNEGOCIO'   => $nrParcNegocio,
            'P_NRINTELETRDIGIT' => $nrIntegracao,
            'P_DSPATH'          => $dsPath,
            'P_CDOPERADOR'      => $cdOperador
        ];

        if($isAceiteGestor == 'S') {
            $this->certisignRepository->aprovaAssinaturaGestor(
                $params
            );
            return;
        }

        $this->certisignRepository->aprovaAssinaturaVinculo(
            $params
        );
    }

    public function reprovaAssinatura(
        $nrIntegracao, $nrParcNegocio, $cdOperador, $isAceiteGestor, $dsRejeicao
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRPARCNEGOCIO'   => $nrParcNegocio,
            'P_NRINTELETRDIGIT' => $nrIntegracao,
            'P_DSREJEICAO'      => $dsRejeicao,
            'P_CDOPERADOR'      => $cdOperador
        ];

        if($isAceiteGestor == 'S') {
            $this->certisignRepository->reprovaAssinaturaGestor(
                $params
            );
            return;
        }

        $this->certisignRepository->reprovaAssinaturaVinculo(
            $params
        );
    }

    public function getIntegracaoDigitalFile(
        int $nrVinculo,
        int $nrParcNegocio,
        int $nrIntegracaoDigital
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $paramsAssi = [
            'P_NRORG'           => $nrOrg,
            'P_NRPARCNEGOCIO'   => $nrParcNegocio,
            'P_NRINTELETRDIGIT' => $nrIntegracaoDigital
        ];

        $docAssinatura = $this->certisignRepository->getAssinaturaRelatorio(
            $paramsAssi
        );

        if (!$docAssinatura) {
            return '';
        }

        $path = isset($docAssinatura['DSPATH']) ?
            $docAssinatura['DSPATH'] :
            '';

        if ($path === '') {
            return '';
        }

        $file = $this->utilities->downloadFile($path);

        return $file;
    }
}